@@ -43,7 +43,7 @@ test_that("output has correct structure", {
   expect_s3_class(foo$fit_mx$M1, "rq")
   expect_s3_class(foo$fit_mx$M2, "rq")
   expect_s3_class(foo$fit_ymx, "rq")
-  expect_null(foo$fit$fit_yx)
+  expect_null(foo$fit_yx)
 
 })
 
@@ -56,6 +56,7 @@ test_that("arguments are correctly passed", {
   expect_identical(foo$covariates, c("C1", "C2"))
   # robust fit
   expect_identical(foo$robust, "median")
+  expect_identical(foo$family, "gaussian")
   expect_null(foo$control)
 
 })
@@ -113,7 +114,7 @@ test_that("summary returns original object", {
 test_that("object returned by setup_ellipse_plot() has correct structure", {
 
   # not meaningful for median regression
-  expect_error(setup_ellipse_plot(boot))
+  expect_error(setup_ellipse_plot(foo))
 
 })
 
