@@ -24,24 +24,26 @@ C2 <- rnorm(n)
 test_data <- data.frame(X, Y, M1, M2, C1, C2)
 
 ## run bootstrap test and compute summary
-level <- 0.9
+level <- c(0.9, 0.95)
 ctrl <- cov_control(prob = 0.9)
 set.seed(seed)
 boot <- test_mediation(test_data, x = "X", y = "Y", m = "M1",
-                       test = "boot", R = R, level = level, type = "bca",
+                       test = "boot", R = R, level = level[1], type = "bca",
                        method = "covariance", robust = TRUE, control = ctrl)
 
 ## compute summary
 summary_boot <- summary(boot, type = "boot")
 summary_data <- summary(boot, type = "data")
 
+## retest with different parameters
+boot_less <- retest(boot, alternative = "less", level = level[2])
+boot_greater <- retest(boot, alternative = "greater", level = level[2])
+boot_perc <- retest(boot, type = "perc", level = level[2])
+
 ## create data for plotting
-ci <- setup_ci_plot(boot, level = level)
-density <- setup_density_plot(boot, level = level)
+ci <- setup_ci_plot(boot)
+density <- setup_density_plot(boot)
 ellipse <- setup_ellipse_plot(boot)
-# deprecated:
-dot_deprecated <- suppressWarnings(fortify(boot, method = "dot"))
-density_deprecated <- suppressWarnings(fortify(boot, method = "density"))
 
 ## stuff needed to check correctness
 coef_names <- c("a", "b", "Direct", "Total", "ab")
@@ -69,7 +71,7 @@ test_that("arguments are correctly passed", {
   # number of bootstrap replicates
   expect_identical(boot$R, as.integer(R))  # doesn't hold for too many outliers
   # confidence level
-  expect_identical(boot$level, level)
+  expect_identical(boot$level, level[1])
   # type of confidence intervals
   expect_identical(boot$type, "bca")
   # variable names
@@ -259,6 +261,72 @@ test_that("effect summaries contain correct coefficient values", {
 
 })
 
+test_that("output of retest() has correct structure", {
+
+  # bootstrap test
+  expect_identical(class(boot_less), class(boot))
+  expect_identical(class(boot_greater), class(boot))
+  expect_identical(class(boot_perc), class(boot))
+  # regression fit
+  expect_identical(boot_less$fit, boot$fit)
+  expect_identical(boot_greater$fit, boot$fit)
+  expect_identical(boot_perc$fit, boot$fit)
+  # bootstrap replicates
+  expect_identical(boot_less$reps, boot$reps)
+  expect_identical(boot_greater$reps, boot$reps)
+  expect_identical(boot_perc$reps, boot$reps)
+
+})
+
+test_that("arguments of retest() are correctly passed", {
+
+  # alternative hypothesis
+  expect_identical(boot_less$alternative, "less")
+  expect_identical(boot_greater$alternative, "greater")
+  expect_identical(boot_perc$alternative, "twosided")
+  # confidence level
+  expect_identical(boot_less$level, level[2])
+  expect_identical(boot_greater$level, level[2])
+  expect_identical(boot_perc$level, level[2])
+  # type of confidence intervals
+  expect_identical(boot_less$type, "bca")
+  expect_identical(boot_greater$type, "bca")
+  expect_identical(boot_perc$type, "perc")
+  # indirect effect
+  expect_identical(boot_less$ab, boot$ab)
+  expect_identical(boot_greater$ab, boot$ab)
+  expect_identical(boot_perc$ab, boot$ab)
+  # confidence interval for alternative = "less"
+  expect_identical(length(boot_less$ci), length(boot$ci))
+  expect_equivalent(boot_less$ci[1], -Inf)
+  expect_equal(boot_less$ci[2], boot$ci[2])
+  expect_identical(colnames(confint(boot_less)), c("Lower", "Upper"))
+  # confidence interval for alternative = "greater"
+  expect_identical(length(boot_greater$ci), length(boot$ci))
+  expect_equal(boot_greater$ci[1], boot$ci[1])
+  expect_equivalent(boot_greater$ci[2], Inf)
+  expect_identical(colnames(confint(boot_greater)), c("Lower", "Upper"))
+  # confidence interval for type = "perc"
+  expect_identical(length(boot_perc$ci), length(boot$ci))
+
+})
+
+test_that("output of p_value() method has correct attributes", {
+
+  digits <- 3
+  p_boot <- p_value(boot_perc, type = "boot", digits = digits)
+  p_data <- p_value(boot_perc, type = "data", digits = digits)
+  # bootstrapped effects
+  expect_length(p_boot, 5L)
+  expect_named(p_boot, coef_names)
+  expect_equal(p_boot["ab"], round(p_boot["ab"], digits = digits))
+  # effects computed on original sample
+  expect_length(p_data, 5L)
+  expect_named(p_data, coef_names)
+  expect_equal(p_data["ab"], round(p_data["ab"], digits = digits))
+
+})
+
 test_that("objects returned by setup_xxx_plot() have correct structure", {
 
   ## ci plot
@@ -273,7 +341,7 @@ test_that("objects returned by setup_xxx_plot() have correct structure", {
   effect_names <- c("Direct", "ab")
   expect_identical(ci$ci$Effect, factor(effect_names, levels = effect_names))
   # check confidence level
-  expect_identical(ci$level, level)
+  expect_identical(ci$level, level[1])
   # check logical for multiple methods
   expect_false(ci$have_methods)
 
@@ -296,7 +364,7 @@ test_that("objects returned by setup_xxx_plot() have correct structure", {
   # check type of test
   expect_identical(density$test, "boot")
   # check confidence level
-  expect_identical(density$level, level)
+  expect_identical(density$level, level[1])
   # check logical for multiple effects
   expect_false(density$have_effect)
   # check logical for multiple methods
@@ -308,67 +376,6 @@ test_that("objects returned by setup_xxx_plot() have correct structure", {
 })
 
 
-## deprecated:
-
-test_that("data returned by fortify() has correct structure", {
-
-  ## dot plot
-  # check dimensions
-  expect_s3_class(dot_deprecated, "data.frame")
-  expect_identical(dim(dot_deprecated), c(2L, 4L))
-  # check column names
-  column_names <- c("Effect", "Point", "Lower", "Upper")
-  expect_named(dot_deprecated, column_names)
-  # check that direct effect and indirect effect are plotted by default
-  effect_names <- c("Direct", "ab")
-  expect_identical(dot_deprecated$Effect, factor(effect_names, levels = effect_names))
-
-  ## density plot
-  # check dimensions
-  expect_s3_class(density_deprecated, "data.frame")
-  expect_identical(ncol(density_deprecated), 2L)
-  expect_gt(nrow(density_deprecated), 0L)
-  # check column names
-  column_names <- c("ab", "Density")
-  expect_named(density_deprecated, column_names)
-
-})
-
-test_that("data returned by fortify() has correct attributes", {
-
-  ## dot plot
-  # check aesthetic mapping
-  mapping <- aes_string(x = "Effect", y = "Point",
-                        ymin = "Lower", ymax = "Upper")
-  expect_equal(attr(dot_deprecated, "mapping"), mapping)
-  # check default geom()
-  expect_identical(attr(dot_deprecated, "geom"), geom_pointrange)
-  # check facets
-  expect_null(attr(dot_deprecated, "facets"))
-  # check that method is stored correctly
-  expect_identical(attr(dot_deprecated, "method"), "dot")
-
-  ## density plot
-  # check aesthetic mapping
-  mapping <- aes_string(x = "ab", y = "Density")
-  expect_equal(attr(density_deprecated, "mapping"), mapping)
-  # check default geom()
-  expect_equal(attr(density_deprecated, "geom"), robmed:::geom_densityline)
-  # check facets
-  expect_null(attr(density_deprecated, "facets"))
-  # check title
-  expect_identical(attr(density_deprecated, "main"), "Bootstrap distribution")
-  # check confidence interval
-  ci <- attr(density_deprecated, "ci")
-  expect_s3_class(ci, "data.frame")
-  expect_identical(dim(ci), c(1L, 4L))
-  expect_named(ci, c("ab", "Density", "Lower", "Upper"))
-  # check that method is stored correctly
-  expect_identical(attr(density_deprecated, "method"), "density")
-
-})
-
-
 # ## only implemented for simple mediation without covariates
 #
 # test_that("covariates not implemented", {
@@ -378,7 +385,7 @@ test_that("data returned by fortify() has correct attributes", {
 #   suppressWarnings(
 #     test_reg <- test_mediation(test_data, x = "X", y = "Y", m = "M1",
 #                                covariates = c("C1", "C2"), test = "boot",
-#                                R = 500, level = level, type = "perc",
+#                                R = 500, level = level[1], type = "perc",
 #                                method = "regression", robust = TRUE)
 #   )
 #
@@ -387,7 +394,7 @@ test_that("data returned by fortify() has correct attributes", {
 #   expect_warning(
 #     test_cov <- test_mediation(test_data, x = "X", y = "Y", m = "M1",
 #                                covariates = c("C1", "C2"), test = "boot",
-#                                R = 500, level = level, type = "perc",
+#                                R = 500, level = level[1], type = "perc",
 #                                method = "covariance", robust = TRUE)
 #   )
 #
@@ -402,7 +409,7 @@ test_that("data returned by fortify() has correct attributes", {
 #   set.seed(seed)
 #   suppressWarnings(
 #     test_reg <- test_mediation(test_data, x = "X", y = "Y", m = c("M1", "M2"),
-#                                test = "boot", R = 500, level = level,
+#                                test = "boot", R = 500, level = level[1],
 #                                type = "perc", method = "regression",
 #                                robust = TRUE)
 #   )
@@ -411,7 +418,7 @@ test_that("data returned by fortify() has correct attributes", {
 #   set.seed(seed)
 #   expect_warning(
 #     test_cov <- test_mediation(test_data, x = "X", y = "Y", m = c("M1", "M2"),
-#                                test = "boot", R = 500, level = level,
+#                                test = "boot", R = 500, level = level[1],
 #                                type = "perc", method = "covariance",
 #                                robust = TRUE)
 #   )
